"""
This is the celtux.distro module, it abstracts the 'distro' section of an
fll.config.Config object. It provides a hook for distro specific actions
and includes (for now) celtux specific actions.

Author:    Niall Walsh
Copyright: Copyright (C) 2013 Niall Walsh <niallwalsh@users.berlios.de>
License:   GPL-2
"""

import os
import shutil

class DistroError(Exception):
    pass


class Distro(object):
    """
    A class for distro preparing a chroot.

    Options  Type                Description
    --------------------------------------------------------------------------
    chroot - (fll.chroot.Chroot) fll.chroot.Chroot object
    config - (dict)              the 'distro' section of fll.config.Config object
    """
    def __init__(self, chroot=None, config={}):
        if chroot is None:
            raise DistroError('must specify chroot=')
        if not config:
            raise DistroError('must specify config=')

        self.chroot = chroot
        self.config = config

    def run(self):
        self.localise()
        self.password()
        self.create_file('/etc/hostname')
        self.create_file('/etc/hosts')

    def password(self):
        """Setup password(s) in the chroot or passwordless access"""
        pwset = 0
        for user in [ u[13:] for u in self.config
                                 if u[:13]=='FLL_PASSWORD_' and len(u)>13 ]:
            if not user == 'root':
                cmd = [ 'adduser' , '--disabled-password',  '--gecos', 'Test User,,,', user ]
                self.chroot.cmd(cmd)
            cmd = [ 'usermod', '-p', self.config['distro'][ 'FLL_PASSWORD_%s' % user ], user ]
            self.chroot.cmd(cmd)
            pwset = pwset + 1
        if pwset == 0 :
            # if no passwords given, hack inittab to autologin tty1 for root
            cmd = [ 'sed', '-i.orig', '-r', 's/(^1\:2.*getty)(.*tty1.*)/\\1 -a root\\2/', '/etc/inittab' ]
            self.chroot.cmd(cmd)
            # our config isn't leant to a 'serial' to enable/disable this :-/
            cmd = [ 'sed', '-i.orig', '-r', 's/^\#(T0\:2.*getty)(.*ttyS0).*/\\1 -a root\\2 115200 vt100/', '/etc/inittab' ]
            self.chroot.cmd(cmd)

    def localise(self):
        """Set the default locale in the chroot"""
        tz = "Etc/UTC"
        if 'FLL_LOCALE_TZ' in self.config:
            tz = self.config['FLL_LOCALE_TZ']
        fh = open( os.path.join(self.chroot.rootdir,'etc/timezone' ), 'w' )
        print >>fh, """\
%s""" % tz
        os.remove( os.path.join(self.chroot.rootdir,'etc/localtime') )
        shutil.copyfile(os.path.join( self.chroot.rootdir,'usr/share/zoneinfo/%s' % tz ) , os.path.join(self.chroot.rootdir,'etc/localtime'))
        fh = open( os.path.join(self.chroot.rootdir,'etc/adjtime' ), 'w' )
        if tz == 'Etc/UTC' or 'FLL_UTC' in self.config:
            print >>fh, """\
0.000000 0 0.000000
0
UTC
"""
        else:
            print >>fh, """\
0.000000 0 0.000000
0
LOCAL

"""
        fh.close()
        lang = 'en_US.utf-8'
        if 'FLL_LOCALE_LANG' in self.config:
            lang = self.config['FLL_LOCALE_LANG']
        cmd = [ 'echo', 'locales locales/default_environment_locale select %s' % lang, '|', 'debconf-set-selections' ]
        self.chroot.cmd(cmd)
        cmd = [ 'update-locale', 'LANG=%s' % lang ]
        fh = open (os.path.join(self.chroot.rootdir, 'etc/default/console-setup'), 'w')
        cs = dict([('CHARMAP','UTF-8'),('CODESET','Uni2'),('FONTFACE','VGA'),('FONTSIZE','16')])
        for name in cs:
            if 'FLL_LOCALE_%s' % name in self.config:
                cs[name]=self.config['FLL_LOCALE_%s' % name]
        print >>fh, """\
ACTIVE_CONSOLES="/dev/tty[1-6]"
CHARMAP="%s"
CODESET="%s"
FONTFACE="%s"
FONTSIZE="%s"
""" % (cs['CHARMAP'], cs['CODESET'], cs['FONTFACE'], cs['FONTSIZE'])
        fh.close()
        kc = dict([('XKBMODEL','linux'),('XKBLAYOUT','us'),('XKBVARIANT',''),('XKBOPTIONS','lv3:ralt_switch,compose:lwin,grp:alt_shift_toggle')])
        for name in kc:
            if "FLL_%s" % name in self.config:
                kc[name]=self.config["FLL_%s" % name]
            if os.path.exists(self.chroot.chroot_path('/etc/default/keyboard')):
                cmd = [ 'sed', '-i', '-e', 's/^\(%s\=\).*/\\1\"%s\"/' % (name, kc[name]), '/etc/default/keyboard' ]
                self.chroot.cmd(cmd)
        if os.path.exists(self.chroot.chroot_path('/bin/setupcon')):
            cmd = [ 'setupcon', '--save-only' ]
            self.chroot.cmd(cmd)

    def create_file(self, filename, mode=0644):
        fh = None
        try:
            fh = open(self.chroot.chroot_path(filename), 'w')

            if filename == '/etc/hostname':
                if not 'FLL_HOSTNAME' in self.config:
                    self.config['FLL_HOSTNAME']='chroot'
                print >>fh, self.config['FLL_HOSTNAME']

            elif filename == '/etc/hosts':
                print >>fh, """\
127.0.0.1\tlocalhost
127.0.0.1\t%s

# Below lines are for IPv6 capable hosts
::1     ip6-localhost ip6-loopback
fe00::0 ip6-localnet
ff00::0 ip6-mcastprefix
ff02::1 ip6-allnodes
ff02::2 ip6-allrouters
ff02::3 ip6-allhosts""" % self.config['FLL_HOSTNAME']
        except IOError:
            raise ChrootError('failed to write: ' + filename)
        finally:
            if fh:
                fh.close()
                os.chmod(self.chroot.chroot_path(filename), mode)
