"""
This is the sunxi.chroot module, it provides a class for bootstrapping
and executing commands within a chroot.

Author:    Niall Walsh
Copyright: Copyright (C) 2013 Niall Walsh <niallwalsh@users.berlios.de>
License:   GPL-2
"""

import fll.misc
import os
import subprocess
import shlex
import shutil
import celtux.chroot

class ChrootError(celtux.chroot.ChrootError):
    """
    An Error class for use by Chroot.
    """
    pass


class Chroot(celtux.chroot.Chroot):
    """
    A class which provides the ability to bootstrap and execute commands
    within a chroot.

    Options        Type   Description
    --------------------------------------------------------------------------
    rootdir      - (str)  path to root of chroot
    architecture - (str)  architecture of chroot
    config       - (dict) the 'chroot' section of fll.config.Config object
    """

    def bootstrap(self):
        """Bootstrap a Debian chroot. By default it will bootstrap a minimal
        sid chroot with cdebootstrap."""
        utility = self.config['bootstrap']['utility']
        uri = self.config['bootstrap']['uri']
        suite = self.config['bootstrap']['suite']
        flavour = self.config['bootstrap']['flavour']
        include = self.config['bootstrap']['include']
        exclude = self.config['bootstrap']['exclude']

        # Use chroot verbosity level.
        quiet = self.config['quiet']
        # default verbosity level is noisy enough imo
        verbose = False
        debug = self.config['debug']

        cmd = [utility]

        if utility == 'cdebootstrap':
            cmd.append('--flavour=' + flavour)
        elif utility == 'debootstrap':
            if flavour == 'minimal':
                flavour = 'minbase'
            elif flavour == 'build':
                flavour = 'buildd'
            cmd.append('--variant=' + flavour)
        else:
            raise ChrootError('unknown utility: ' + utility)

        cmd.append('--arch=' + self.architecture)
        if self.config['foreign']:
            cmd.append('--foreign')

        if include:
            cmd.append('--include=' + include)
        if exclude:
            cmd.append('--exclude=' + exclude)
        if verbose:
            cmd.append('--verbose')        
        if debug and utility == 'cdebootstrap':
            cmd.append('--debug')
        if quiet and utility == 'cdebootstrap':
            cmd.append('--quiet')
        
        cmd.append(suite)
        cmd.append(self.rootdir)
        cmd.append(uri)

        try:
            fll.misc.cmd(cmd)
        except OSError:
            raise ChrootError('bootstrap command failed: %s' % ' '.join(cmd))

        if self.config['foreign']:
            if utility == 'cdebootstrap':
                cmd = [ '/sbin/cdebootstrap-foreign' ]
            else:
                cmd = [ '/debootstrap/debootstrap' ]
            helper = os.path.join(self.rootdir,cmd[0][1:])
            if os.path.exists(helper):
                helperarch = self.architecture
                if helperarch == 'armhf':
                    helperarch = 'arm'
                handler = '/usr/bin/qemu-%s-static' % helperarch
                try:
                    shutil.copy2(handler,os.path.join(self.rootdir,handler[1:]))
                except OSError:
                    raise ChrootError('failed to copy %s to rootfs for --foreign' % handler)
                try:
                    self.cmd(cmd,ptsonly=True)
                except OSError:
                    raise ChrootError('failed to run %s for --foreign rootfs' % cmd)
            else:
                raise ChrootError('failed to find second stage bootstrapper %s%s for --foreign rootfs' % (self.rootdir,  cmd[0]) )

        # Some flavours use cdebootstrap-helper-rc.d, some don't. We'll
        # impliment our our own policy-rc.d for consistency.
        if utility == 'cdebootstrap':
            self.cmd('dpkg --purge cdebootstrap-helper-rc.d'.split(),
                     silent=self.config['quiet'])

    def mountvirtfs(self,ptsonly=False):
        """Mount /sys, /proc, /dev/pts virtual filesystems in the chroot."""
        virtfs = {'devpts': '/dev/pts', 'proc': '/proc', 'sysfs': '/sys'}
        if ptsonly:
            virtfs = {'devpts': '/dev/pts'}
        for vfstype, mnt in virtfs.items():
            if os.path.exists(self.chroot_path(mnt)) is False:
                os.mkdir(self.chroot_path(mnt))
            cmd = ['mount', '-t', vfstype, 'none', self.chroot_path(mnt)]
            try:
                subprocess.check_call(cmd, preexec_fn=fll.misc.restore_sigpipe)
            except (subprocess.CalledProcessError, OSError):
                raise ChrootError('failed to mount virtfs: ' + mnt)

    def cmd(self, cmd, pipe=False, quiet=False, silent=False, ptsonly=False):
        """Execute a command in the chroot."""
        if isinstance(cmd, str):
            cmd = shlex.split(cmd)

        if silent is False:
            print 'CHROOT %s %s' % (self.rootdir, ' '.join(cmd))

        devnull = output = None

        if quiet is False:
            quiet = self.config['quiet']

        self.mountvirtfs(ptsonly=ptsonly)
        try:
            if pipe:
                proc = subprocess.Popen(cmd, preexec_fn=self._chroot, cwd='/',
                                        stdout=subprocess.PIPE)
                output = proc.communicate()[0]
            elif quiet or silent:
                devnull = os.open(os.devnull, os.O_RDWR)
                proc = subprocess.Popen(cmd, preexec_fn=self._chroot, cwd='/',
                                        stdout=devnull)
                proc.wait()
            else:
                proc = subprocess.Popen(cmd, preexec_fn=self._chroot, cwd='/')
                proc.wait()
        except OSError, e:
            raise ChrootError('chrooted command failed: %s' % e)
        finally:
            self.umountvirtfs()
            if devnull:
                os.close(devnull)

        if proc.returncode != 0:
            raise ChrootError('chrooted command returncode=%d: %s' %
                              (proc.returncode, ' '.join(cmd)))

        if pipe:
            return output
